"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const operators_1 = require("rxjs/operators");
const v4_1 = __importDefault(require("uuid/v4"));
const protocol_1 = require("../../protocol");
const common_1 = require("./common");
/**
 * Support for configuration settings managed by the client (workspace/didChangeConfiguration notifications to the
 * server).
 *
 * @template C configuration cascade type
 */
class ConfigurationChangeNotificationFeature {
    constructor(client, configurationCascade) {
        this.client = client;
        this.configurationCascade = configurationCascade;
        this.subscriptionsByID = new Map();
        this.messages = protocol_1.DidChangeConfigurationNotification.type;
    }
    fillInitializeParams(params) {
        // This runs synchronously because this.configurationCascade's root source is a BehaviorSubject (which has
        // an initial value). Confirm it is synchronous just in case, because a bug here would be hard to diagnose.
        let sync = false;
        this.configurationCascade
            .pipe(operators_1.first())
            .subscribe(configurationCascade => {
            params.configurationCascade = configurationCascade;
            sync = true;
        })
            .unsubscribe();
        if (!sync) {
            throw new Error('configuration is not immediately available');
        }
    }
    fillClientCapabilities(capabilities) {
        common_1.ensure(common_1.ensure(capabilities, 'workspace'), 'didChangeConfiguration').dynamicRegistration = true;
    }
    initialize(capabilities) {
        this.register(this.messages, { id: v4_1.default(), registerOptions: undefined });
    }
    register(message, data) {
        if (this.subscriptionsByID.has(data.id)) {
            throw new Error(`registration already exists with ID ${data.id}`);
        }
        this.subscriptionsByID.set(data.id, this.configurationCascade.subscribe(configurationCascade => this.client.sendNotification(protocol_1.DidChangeConfigurationNotification.type, { configurationCascade })));
    }
    unregister(id) {
        const sub = this.subscriptionsByID.get(id);
        if (!sub) {
            throw new Error(`no registration with ID ${id}`);
        }
        this.subscriptionsByID.delete(id);
    }
    unregisterAll() {
        for (const sub of this.subscriptionsByID.values()) {
            sub.unsubscribe();
        }
        this.subscriptionsByID.clear();
    }
}
exports.ConfigurationChangeNotificationFeature = ConfigurationChangeNotificationFeature;
/**
 * Support for the server requesting the client's configuration (workspace/configuration request to the client).
 *
 * @template C configuration cascade type
 */
class ConfigurationFeature {
    constructor(client, configurationCascade) {
        this.client = client;
        this.configurationCascade = configurationCascade;
    }
    fillClientCapabilities(capabilities) {
        capabilities.workspace = capabilities.workspace || {};
        capabilities.workspace.configuration = true;
    }
    initialize() {
        this.client.onRequest(protocol_1.ConfigurationRequest.type, (params, token) => {
            const configuration = params => this.configurationCascade
                .pipe(operators_1.first(), operators_1.map(configurationCascade => {
                const result = [];
                for (const item of params.items) {
                    result.push(this.getConfiguration(configurationCascade, item.scopeUri, item.section !== null ? item.section : undefined));
                }
                return result;
            }))
                .toPromise();
            return configuration(params, token);
        });
    }
    getConfiguration(configurationCascade, resource, section) {
        if (resource) {
            throw new Error('configuration request: resource param is not supported');
        }
        if (section) {
            throw new Error('configuration request: section param is not supported');
        }
        // TODO(sqs): Support only returning partial settings (based on the resource/section args).
        // Also figure out in what cases it's OK for one extension to see another extension's
        // settings. In some cases this is dangerous because it would let extensions see the access
        // tokens, etc., configured for other extensions.
        return configurationCascade;
    }
}
exports.ConfigurationFeature = ConfigurationFeature;
/**
 * Support for the server updating the client's configuration (configuration/update request to the
 * client).
 */
class ConfigurationUpdateFeature {
    constructor(client, 
    /** Called when the client receives a configuration/update request. */
    update) {
        this.client = client;
        this.update = update;
    }
    fillClientCapabilities(capabilities) {
        common_1.ensure(capabilities, 'configuration').update = true;
    }
    initialize() {
        this.client.onRequest(protocol_1.ConfigurationUpdateRequest.type, this.update);
    }
}
exports.ConfigurationUpdateFeature = ConfigurationUpdateFeature;
//# sourceMappingURL=configuration.js.map