"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const v4_1 = __importDefault(require("uuid/v4"));
const protocol_1 = require("../../protocol");
const common_1 = require("./common");
/**
 * Support for requests that retrieve a list of locations (e.g., textDocument/definition,
 * textDocument/implementation, and textDocument/typeDefinition).
 */
class TextDocumentLocationFeature extends common_1.Feature {
    constructor(client, registry) {
        super(client);
        this.registry = registry;
    }
    initialize(capabilities, documentSelector) {
        if (!this.isSupported(capabilities) || !documentSelector) {
            return;
        }
        this.register(this.messages, {
            id: v4_1.default(),
            registerOptions: { documentSelector },
        });
    }
    registerProvider(options) {
        const client = this.client;
        const provideTextDocumentLocation = params => client.sendRequest(this.messages, params);
        const middleware = this.getMiddleware ? this.getMiddleware(client.options.middleware) : undefined;
        return this.registry.registerProvider(options, (params) => middleware ? middleware(params, provideTextDocumentLocation) : provideTextDocumentLocation(params));
    }
}
exports.TextDocumentLocationFeature = TextDocumentLocationFeature;
/**
 * Support for definition requests (textDocument/definition requests to the server).
 */
class TextDocumentDefinitionFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = protocol_1.DefinitionRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'definition');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.definitionProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentDefinition;
    }
}
exports.TextDocumentDefinitionFeature = TextDocumentDefinitionFeature;
/**
 * Support for implementation requests (textDocument/implementation requests to the server).
 */
class TextDocumentImplementationFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = protocol_1.ImplementationRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'implementation');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.implementationProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentImplementation;
    }
}
exports.TextDocumentImplementationFeature = TextDocumentImplementationFeature;
/**
 * Support for type definition requests (textDocument/typeDefinition requests to the server).
 */
class TextDocumentTypeDefinitionFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = protocol_1.TypeDefinitionRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'typeDefinition');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.typeDefinitionProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentTypeDefinition;
    }
}
exports.TextDocumentTypeDefinitionFeature = TextDocumentTypeDefinitionFeature;
/**
 * Support for references requests (textDocument/references requests to the server).
 */
class TextDocumentReferencesFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = protocol_1.ReferencesRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'references');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.referencesProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentReferences;
    }
}
exports.TextDocumentReferencesFeature = TextDocumentReferencesFeature;
//# sourceMappingURL=location.js.map