"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const operators_1 = require("rxjs/operators");
const v4_1 = __importDefault(require("uuid/v4"));
const protocol_1 = require("../../protocol");
const textDocument_1 = require("../../types/textDocument");
const common_1 = require("./common");
class TextDocumentNotificationFeature {
    constructor(client, observable, type, middleware, createParams, selectorFilter) {
        this.client = client;
        this.observable = observable;
        this.type = type;
        this.middleware = middleware;
        this.createParams = createParams;
        this.selectorFilter = selectorFilter;
        this.subscription = null;
        this.selectors = new Map();
    }
    register(_message, data) {
        if (!data.registerOptions.documentSelector) {
            return;
        }
        if (this.selectors.has(data.id)) {
            throw new Error(`registration already exists with ID ${data.id}`);
        }
        this.selectors.set(data.id, data.registerOptions.documentSelector);
        if (!this.subscription) {
            this.subscription = this.observable.subscribe(data => this.callback(data));
        }
    }
    callback(data) {
        if (!this.selectorFilter || this.selectorFilter(this.selectors.values(), data)) {
            if (this.middleware) {
                this.middleware(data, data => this.client.sendNotification(this.type, this.createParams(data)));
            }
            else {
                this.client.sendNotification(this.type, this.createParams(data));
            }
            this.notificationSent(data);
        }
    }
    notificationSent(_data) {
        /* noop */
    }
    unregister(id) {
        if (!this.selectors.delete(id)) {
            throw new Error(`no registration with ID ${id}`);
        }
        this.selectors.delete(id);
        if (this.selectors.size === 0 && this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    }
    unregisterAll() {
        this.selectors.clear();
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    }
}
exports.TextDocumentNotificationFeature = TextDocumentNotificationFeature;
/**
 * Support for notifying the server of when the client opened a text document (textDocument/didOpen notifications
 * sent by the client to the server).
 */
class TextDocumentDidOpenFeature extends TextDocumentNotificationFeature {
    constructor(client, environmentTextDocument) {
        super(client, environmentTextDocument.pipe(operators_1.filter((v) => v !== null)), protocol_1.DidOpenTextDocumentNotification.type, client.options.middleware.didOpen, textDocument => ({
            textDocument,
        }), textDocument_1.match);
    }
    get messages() {
        return protocol_1.DidOpenTextDocumentNotification.type;
    }
    fillClientCapabilities(capabilities) {
        common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'synchronization').dynamicRegistration = true;
    }
    initialize(capabilities, documentSelector) {
        const textDocumentSyncOptions = resolveTextDocumentSync(capabilities.textDocumentSync);
        if (documentSelector && textDocumentSyncOptions && textDocumentSyncOptions.openClose) {
            this.register(this.messages, {
                id: v4_1.default(),
                registerOptions: { documentSelector },
            });
        }
    }
}
exports.TextDocumentDidOpenFeature = TextDocumentDidOpenFeature;
/**
 * Support for notifying the server of when the client closed a text document (textDocument/didClose notifications
 * sent by the client to the server).
 */
class TextDocumentDidCloseFeature extends TextDocumentNotificationFeature {
    constructor(client, environmentTextDocument) {
        super(client, environmentTextDocument.pipe(operators_1.bufferCount(2, 1), 
        // When the previous value emitted was a document, it means the most recent value was either a
        // different document or null. In both cases, it means the previous document is closed.
        operators_1.filter((v) => v[0] !== null), operators_1.map(([closedDocument]) => closedDocument)), protocol_1.DidCloseTextDocumentNotification.type, client.options.middleware.didClose, textDocument => ({
            textDocument: { uri: textDocument.uri },
        }), textDocument_1.match);
    }
    get messages() {
        return protocol_1.DidCloseTextDocumentNotification.type;
    }
    fillClientCapabilities(capabilities) {
        common_1.ensure(common_1.ensure(capabilities, 'textDocument'), 'synchronization').dynamicRegistration = true;
    }
    initialize(capabilities, documentSelector) {
        const textDocumentSyncOptions = resolveTextDocumentSync(capabilities.textDocumentSync);
        if (documentSelector && textDocumentSyncOptions && textDocumentSyncOptions.openClose) {
            this.register(this.messages, {
                id: v4_1.default(),
                registerOptions: { documentSelector },
            });
        }
    }
}
exports.TextDocumentDidCloseFeature = TextDocumentDidCloseFeature;
function resolveTextDocumentSync(value) {
    if (!value) {
        return undefined;
    }
    else if (typeof value === 'number') {
        return {
            openClose: true,
            change: value,
            save: { includeText: false },
        };
    }
    return value;
}
exports.resolveTextDocumentSync = resolveTextDocumentSync;
//# sourceMappingURL=textDocument.js.map