import { Observable, Unsubscribable } from 'rxjs';
import { Client, ClientOptions } from '../client/client';
import { Trace } from '../jsonrpc2/trace';
import { ConfigurationCascade, ConfigurationUpdateParams, InitializeParams, LogMessageParams, MessageActionItem, ShowInputParams, ShowMessageParams, ShowMessageRequestParams } from '../protocol';
import { Environment, ObservableEnvironment } from './environment';
import { Extension } from './extension';
import { Registries } from './registries';
/** The minimal unique identifier for a client. */
export interface ClientKey extends Pick<InitializeParams, 'root'> {
    id: string;
}
/** A client and its unique identifier (key). */
export interface ClientEntry {
    client: Client;
    key: ClientKey;
}
/** The source of a message (used with LogMessageParams, ShowMessageParams, ShowMessageRequestParams). */
interface MessageSource {
    /** The ID of the extension that produced this log message. */
    extension: string;
}
interface PromiseCallback<T> {
    resolve: (p: T | Promise<T>) => void;
}
declare type ShowMessageRequest = ShowMessageRequestParams & MessageSource & PromiseCallback<MessageActionItem | null>;
declare type ShowInputRequest = ShowInputParams & MessageSource & PromiseCallback<string | null>;
declare type ConfigurationUpdate = ConfigurationUpdateParams & MessageSource & PromiseCallback<void>;
/**
 * Options for creating the controller.
 *
 * @template X extension type
 * @template C configuration cascade type
 */
export interface ControllerOptions<X extends Extension, C extends ConfigurationCascade> {
    /** Returns additional options to use when creating a client. */
    clientOptions: (key: ClientKey, options: ClientOptions, extension: X) => Pick<ClientOptions, 'middleware' | 'createMessageTransports' | 'errorHandler' | 'initializationFailedHandler' | 'trace' | 'tracer' | 'experimentalClientCapabilities'>;
    /**
     * Called before applying the next environment in Controller#setEnvironment. It should have no side effects.
     */
    environmentFilter?: (nextEnvironment: Environment<X, C>) => Environment<X, C>;
}
/**
 * The controller for the environment.
 *
 * @template X extension type
 * @template C configuration cascade type
 */
export declare class Controller<X extends Extension, C extends ConfigurationCascade> implements Unsubscribable {
    private options;
    private _environment;
    private _clientEntries;
    /** An observable that emits whenever the set of clients managed by this controller changes. */
    readonly clientEntries: Observable<ClientEntry[]>;
    private subscriptions;
    /** The registries for various providers that expose extension functionality. */
    readonly registries: Registries<X, C>;
    private readonly _logMessages;
    private readonly _showMessages;
    private readonly _showMessageRequests;
    private readonly _showInputs;
    private readonly _configurationUpdates;
    /** Log messages from extensions. */
    readonly logMessages: Observable<LogMessageParams & MessageSource>;
    /** Messages from extensions intended for display to the user. */
    readonly showMessages: Observable<ShowMessageParams & MessageSource>;
    /** Messages from extensions requesting the user to select an action. */
    readonly showMessageRequests: Observable<ShowMessageRequest>;
    /** Messages from extensions requesting text input from the user. */
    readonly showInputs: Observable<ShowInputRequest>;
    /** Configuration updates from extensions. */
    readonly configurationUpdates: Observable<ConfigurationUpdate>;
    constructor(options: ControllerOptions<X, C>);
    /**
     * Detect when setEnvironment is called within a setEnvironment call, which probably means there is a bug.
     */
    private inSetEnvironment;
    setEnvironment(nextEnvironment: Environment<X, C>): void;
    private onEnvironmentChange;
    private registerClientFeatures;
    readonly environment: ObservableEnvironment<X, C>;
    trace: Trace;
    unsubscribe(): void;
}
export {};
