"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const client_1 = require("../client/client");
const command_1 = require("../client/features/command");
const configuration_1 = require("../client/features/configuration");
const context_1 = require("../client/features/context");
const contribution_1 = require("../client/features/contribution");
const decoration_1 = require("../client/features/decoration");
const hover_1 = require("../client/features/hover");
const location_1 = require("../client/features/location");
const logMessage_1 = require("../client/features/logMessage");
const message_1 = require("../client/features/message");
const textDocument_1 = require("../client/features/textDocument");
const util_1 = require("../util");
const context_2 = require("./context/context");
const environment_1 = require("./environment");
const registries_1 = require("./registries");
/**
 * The controller for the environment.
 *
 * @template X extension type
 * @template C configuration cascade type
 */
class Controller {
    constructor(options) {
        this.options = options;
        this._environment = new rxjs_1.BehaviorSubject(environment_1.EMPTY_ENVIRONMENT);
        this._clientEntries = new rxjs_1.BehaviorSubject([]);
        this.subscriptions = new rxjs_1.Subscription();
        this._logMessages = new rxjs_1.Subject();
        this._showMessages = new rxjs_1.Subject();
        this._showMessageRequests = new rxjs_1.Subject();
        this._showInputs = new rxjs_1.Subject();
        this._configurationUpdates = new rxjs_1.Subject();
        /** Log messages from extensions. */
        this.logMessages = this._logMessages;
        /** Messages from extensions intended for display to the user. */
        this.showMessages = this._showMessages;
        /** Messages from extensions requesting the user to select an action. */
        this.showMessageRequests = this._showMessageRequests;
        /** Messages from extensions requesting text input from the user. */
        this.showInputs = this._showInputs;
        /** Configuration updates from extensions. */
        this.configurationUpdates = this._configurationUpdates;
        /**
         * Detect when setEnvironment is called within a setEnvironment call, which probably means there is a bug.
         */
        this.inSetEnvironment = false;
        this.environment = environment_1.createObservableEnvironment(this._environment);
        this.subscriptions.add(() => {
            for (const c of this._clientEntries.value) {
                c.client.unsubscribe();
            }
        });
        this.registries = new registries_1.Registries(this.environment);
    }
    /** An observable that emits whenever the set of clients managed by this controller changes. */
    get clientEntries() {
        return this._clientEntries;
    }
    setEnvironment(nextEnvironment) {
        if (this.inSetEnvironment) {
            throw new Error('setEnvironment may not be called recursively');
        }
        this.inSetEnvironment = true;
        if (this.options.environmentFilter) {
            nextEnvironment = this.options.environmentFilter(nextEnvironment);
        }
        // External consumers don't see context, and their setEnvironment args lack context.
        if (nextEnvironment.context === context_2.EMPTY_CONTEXT) {
            nextEnvironment = Object.assign({}, nextEnvironment, { context: this._environment.value.context });
        }
        if (util_1.isEqual(this._environment.value, nextEnvironment)) {
            this.inSetEnvironment = false;
            return; // no change
        }
        this._environment.next(nextEnvironment);
        this.onEnvironmentChange();
        this.inSetEnvironment = false;
    }
    onEnvironmentChange() {
        const environment = this._environment.value; // new environment
        // Diff clients.
        const newClients = computeClients(environment);
        const nextClients = [];
        const unusedClients = [];
        for (const oldClient of this._clientEntries.value) {
            const newIndex = newClients.findIndex(newClient => util_1.isEqual(oldClient.key, newClient));
            if (newIndex === -1) {
                // Client is no longer needed.
                unusedClients.push(oldClient);
            }
            else {
                // Client already exists. Settings may have changed, but ConfigurationFeature is responsible for
                // notifying the server of configuration changes.
                newClients.splice(newIndex, 1);
                nextClients.push(oldClient);
            }
        }
        // Remove clients that are no longer in use.
        for (const unusedClient of unusedClients) {
            unusedClient.client.unsubscribe();
        }
        // Create new clients.
        for (const key of newClients) {
            // Find the extension that this client is for.
            const extension = environment.extensions.find(x => x.id === key.id);
            if (!extension) {
                throw new Error(`extension not found: ${key.id}`);
            }
            // Construct client.
            const clientOptions = {
                root: key.root,
                documentSelector: ['*'],
                createMessageTransports: null,
            };
            Object.assign(clientOptions, this.options.clientOptions(key, clientOptions, extension));
            const client = new client_1.Client(key.id, key.id, clientOptions);
            // Register client features.
            this.registerClientFeatures(client, this.environment.configuration);
            // Activate client.
            client.activate();
            nextClients.push({
                key,
                client,
            });
        }
        this._clientEntries.next(nextClients);
    }
    registerClientFeatures(client, configuration) {
        client.registerFeature(new configuration_1.ConfigurationChangeNotificationFeature(client, configuration));
        client.registerFeature(new configuration_1.ConfigurationFeature(client, configuration));
        client.registerFeature(new configuration_1.ConfigurationUpdateFeature(client, (params) => new Promise(resolve => this._configurationUpdates.next(Object.assign({}, params, { extension: client.id, resolve })))));
        client.registerFeature(new context_1.ContextFeature(client, (params) => 
        // Set environment manually, not via Controller#setEnvironment, to avoid recursive setEnvironment calls
        // (when this callback is called during setEnvironment's teardown of unused clients).
        this._environment.next(Object.assign({}, this._environment.value, { context: context_2.applyContextUpdate(this._environment.value.context, params.updates) }))));
        client.registerFeature(new contribution_1.ContributionFeature(this.registries.contribution));
        client.registerFeature(new command_1.ExecuteCommandFeature(client, this.registries.commands));
        client.registerFeature(new textDocument_1.TextDocumentDidOpenFeature(client, this.environment.textDocument));
        client.registerFeature(new textDocument_1.TextDocumentDidCloseFeature(client, this.environment.textDocument));
        client.registerFeature(new location_1.TextDocumentDefinitionFeature(client, this.registries.textDocumentDefinition));
        client.registerFeature(new location_1.TextDocumentImplementationFeature(client, this.registries.textDocumentImplementation));
        client.registerFeature(new location_1.TextDocumentReferencesFeature(client, this.registries.textDocumentReferences));
        client.registerFeature(new location_1.TextDocumentTypeDefinitionFeature(client, this.registries.textDocumentTypeDefinition));
        client.registerFeature(new hover_1.TextDocumentHoverFeature(client, this.registries.textDocumentHover));
        client.registerFeature(new decoration_1.TextDocumentDecorationFeature(client, this.registries.textDocumentDecoration));
        client.registerFeature(new logMessage_1.WindowLogMessageFeature(client, (params) => this._logMessages.next(Object.assign({}, params, { extension: client.id }))));
        client.registerFeature(new message_1.WindowShowMessageFeature(client, (params) => this._showMessages.next(Object.assign({}, params, { extension: client.id })), (params) => new Promise(resolve => {
            this._showMessageRequests.next(Object.assign({}, params, { extension: client.id, resolve }));
        }), (params) => new Promise(resolve => {
            this._showInputs.next(Object.assign({}, params, { extension: client.id, resolve }));
        })));
    }
    set trace(value) {
        for (const client of this._clientEntries.value) {
            client.client.trace = value;
        }
    }
    unsubscribe() {
        this.subscriptions.unsubscribe();
    }
}
exports.Controller = Controller;
function computeClients(environment) {
    const clients = [];
    if (!environment.extensions) {
        return clients;
    }
    for (const x of environment.extensions) {
        clients.push({ id: x.id, root: environment.root });
    }
    return clients;
}
//# sourceMappingURL=controller.js.map