"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
/** Manages and executes commands from all extensions. */
class CommandRegistry {
    constructor() {
        this.entries = new rxjs_1.BehaviorSubject([]);
        /** All commands, emitted whenever the set of registered commands changed. */
        this.commands = this.entries;
    }
    registerCommand(entry) {
        // Enforce uniqueness of command IDs.
        for (const e of this.entries.value) {
            if (e.command === entry.command) {
                throw new Error(`command is already registered: ${JSON.stringify(entry.command)}`);
            }
        }
        this.entries.next([...this.entries.value, entry]);
        return {
            unsubscribe: () => {
                this.entries.next(this.entries.value.filter(e => e !== entry));
            },
        };
    }
    executeCommand(params) {
        return executeCommand(this.commandsSnapshot, params);
    }
    /**
     * The current set of commands. Used by callers that do not need to react to commands being registered or
     * unregistered.
     */
    get commandsSnapshot() {
        return this.entries.value;
    }
}
exports.CommandRegistry = CommandRegistry;
/**
 * Executes the command (in the commands list) specified in params.
 *
 * Most callers should use CommandRegistry's getHover method, which uses the registered commands.
 */
function executeCommand(commands, params) {
    const command = commands.find(c => c.command === params.command);
    if (!command) {
        throw new Error(`command not found: ${JSON.stringify(params.command)}`);
    }
    return Promise.resolve(command.run(...(params.arguments || [])));
}
exports.executeCommand = executeCommand;
//# sourceMappingURL=command.js.map