"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const registry_1 = require("./registry");
const util_1 = require("./util");
/** Provides location results from all extensions for definition, implementation, and type definition requests. */
class TextDocumentLocationProviderRegistry extends registry_1.FeatureProviderRegistry {
    getLocation(params) {
        return getLocation(this.providers, params);
    }
}
exports.TextDocumentLocationProviderRegistry = TextDocumentLocationProviderRegistry;
/**
 * Returns an observable that emits all providers' location results whenever any of the last-emitted set of
 * providers emits hovers.
 *
 * Most callers should use the TextDocumentLocationProviderRegistry class, which uses the registered providers.
 */
function getLocation(providers, params) {
    return getLocations(providers, params).pipe(operators_1.map(results => {
        if (results !== null && results.length === 1) {
            return results[0];
        }
        return results;
    }));
}
exports.getLocation = getLocation;
/**
 * Like getLocation, except the returned observable never emits singular values, always either an array or null.
 */
function getLocations(providers, params) {
    return providers.pipe(operators_1.switchMap(providers => {
        if (providers.length === 0) {
            return [null];
        }
        return rxjs_1.combineLatest(providers.map(provider => rxjs_1.from(provider(params))));
    }), operators_1.map(util_1.flattenAndCompact));
}
exports.getLocations = getLocations;
/**
 * Provides reference results from all extensions.
 *
 * Reference results are always an array or null, unlike results from other location providers (e.g., from
 * textDocument/definition), which can be a single item, an array, or null.
 */
class TextDocumentReferencesProviderRegistry extends TextDocumentLocationProviderRegistry {
    /** Gets reference locations from all extensions. */
    getLocation(params) {
        // References are always an array (unlike other locations, which can be returned as L | L[] |
        // null).
        return getLocations(this.providers, params);
    }
}
exports.TextDocumentReferencesProviderRegistry = TextDocumentReferencesProviderRegistry;
//# sourceMappingURL=location.js.map