"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
/** Base class for provider registries for features. */
class FeatureProviderRegistry {
    constructor(initialEntries) {
        this.entries = new rxjs_1.BehaviorSubject([]);
        /** All providers, emitted whenever the set of registered providers changed. */
        this.providers = this.entries.pipe(operators_1.map(providers => providers.map(({ provider }) => provider)));
        if (initialEntries) {
            this.entries.next(initialEntries);
        }
    }
    registerProvider(registrationOptions, provider) {
        const entry = { registrationOptions, provider };
        this.entries.next([...this.entries.value, entry]);
        return {
            unsubscribe: () => {
                this.entries.next(this.entries.value.filter(e => e !== entry));
            },
        };
    }
    /**
     * The current set of providers. Used by callers that do not need to react to providers being registered or
     * unregistered.
     *
     * NOTE: You should usually use the providers property on this class, not providersSnapshot, even when you
     * think you don't need live-updating results. Providers are registered asynchronously after the CXP client
     * connects (or reconnects) to the server. So, the providers list might be empty at the instant you need the
     * results (because the client was just instantiated and is waiting on a network roundtrip before it registers
     * providers, or because there was a temporary network error and the client is reestablishing the connection).
     * By using the providers property, the consumer will get the results it (probably) expects when the client
     * connects and registers the providers.
     */
    get providersSnapshot() {
        return this.entries.value.map(({ provider }) => provider);
    }
}
exports.FeatureProviderRegistry = FeatureProviderRegistry;
/** An empty provider registry, mainly useful in tests and example code. */
class NoopProviderRegistry extends FeatureProviderRegistry {
}
exports.NoopProviderRegistry = NoopProviderRegistry;
//# sourceMappingURL=registry.js.map