"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const connection_1 = require("../jsonrpc2/connection");
const protocol_1 = require("../protocol");
const commands_1 = require("./features/commands");
const configuration_1 = require("./features/configuration");
const context_1 = require("./features/context");
const windows_1 = require("./features/windows");
class ExtensionHandle {
    constructor(rawConnection, initializeParams) {
        this.rawConnection = rawConnection;
        this.initializeParams = initializeParams;
        this.subscription = new rxjs_1.Subscription();
        this.subscription.add(this.rawConnection);
        this.configuration = configuration_1.createExtConfiguration(this, initializeParams.configurationCascade);
        this.windows = windows_1.createExtWindows(this);
        this.commands = commands_1.createExtCommands(this);
        this.context = context_1.createExtContext(this);
    }
    get root() {
        return this.initializeParams.root;
    }
    close() {
        this.subscription.unsubscribe();
    }
}
const consoleLogger = {
    error(message) {
        console.error(message);
    },
    warn(message) {
        console.warn(message);
    },
    info(message) {
        console.info(message);
    },
    log(message) {
        console.log(message);
    },
};
/**
 * Activates a CXP extension by calling its `run` entrypoint function with the CXP API handle as the first
 * argument.
 *
 * @template C the extension's settings
 * @param transports The message reader and writer to use for communication with the client.
 * @param run The extension's `run` entrypoint function.
 * @return A promise that resolves when the extension's `run` function has been called.
 */
function activateExtension(transports, run) {
    const connection = connection_1.createMessageConnection(transports, consoleLogger);
    return new Promise(resolve => {
        let initializationParams;
        connection.onRequest(protocol_1.InitializeRequest.type, params => {
            initializationParams = params;
            return {
                capabilities: {
                    textDocumentSync: { openClose: true },
                    decorationProvider: true,
                },
            };
        });
        connection.onNotification(protocol_1.InitializedNotification.type, () => {
            run(new ExtensionHandle(connection, initializationParams));
            resolve();
        });
        connection.listen();
    });
}
exports.activateExtension = activateExtension;
//# sourceMappingURL=extension.js.map