"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const protocol_1 = require("../../protocol");
const util_1 = require("../../util");
class ExtConfiguration extends rxjs_1.BehaviorSubject {
    constructor(ext, initial) {
        super(initial.merged);
        this.ext = ext;
        this[Symbol.observable] = () => this;
        ext.rawConnection.onNotification(protocol_1.DidChangeConfigurationNotification.type, params => {
            this.next(params.configurationCascade.merged);
        });
    }
    get(key) {
        return this.value[key];
    }
    watch(...keys) {
        return this.pipe(operators_1.distinctUntilChanged((a, b) => keys.every(key => util_1.isEqual(a[key], b[key]))));
    }
    update(key, value) {
        const path = [key];
        // Optimistically apply configuration update locally. If this diverges from the server's state, a
        // subsequent didChangeConfiguration notification will inform us.
        const cur = this.value;
        this.next(setValueAtKeyPath(cur, path, value));
        return this.ext.rawConnection.sendRequest(protocol_1.ConfigurationUpdateRequest.type, {
            path,
            value,
        });
    }
}
/**
 * Returns the source object with the given value inserted in the location specified by the key path. The source
 * object is not modified. The key path indexes into the object successively for each element in the key path.
 *
 * If the value is `undefined`, the value at the key path is removed.
 *
 * This must behave identically to {@link module:jsonc-parser.setProperty}.
 */
function setValueAtKeyPath(source, path, value) {
    if (path.length === 0) {
        // Overwrite entire value.
        return value;
    }
    const root = [source];
    let prev = root; // maintain an lvalue that we can assign to
    for (const [i, key] of path.entries()) {
        const last = i === path.length - 1;
        const prevKey = i === 0 ? 0 : path[i - 1];
        if (typeof key === 'string') {
            if (last) {
                if (value === undefined) {
                    prev[prevKey] = Object.assign({}, prev[prevKey]);
                    delete prev[prevKey][key];
                }
                else if (prev[prevKey] !== null &&
                    typeof prev[prevKey] === 'object' &&
                    !Array.isArray(prev[prevKey])) {
                    prev[prevKey] = Object.assign({}, prev[prevKey], { [key]: value });
                }
                else {
                    prev[prevKey] = { [key]: value };
                }
            }
            else {
                prev[prevKey] =
                    prev[prevKey] !== null && typeof prev[prevKey] === 'object' && !Array.isArray(prev[prevKey])
                        ? Object.assign({}, prev[prevKey]) : {};
            }
        }
        else if (typeof key === 'number') {
            if (last) {
                const index = key === -1 ? prev[prevKey].length : key;
                const head = Array.isArray(prev[prevKey]) ? prev[prevKey].slice(0, index) : [];
                const tail = Array.isArray(prev[prevKey]) ? prev[prevKey].slice(index + 1) : [];
                if (value === undefined) {
                    prev[prevKey] = [...head, ...tail];
                }
                else {
                    prev[prevKey] = [...head, value, ...tail];
                }
            }
            else {
                prev[prevKey] = Array.isArray(prev[prevKey]) ? [...prev[prevKey]] : [];
            }
        }
        else {
            throw new Error(`invalid key in key path: ${key} (full key path: ${JSON.stringify(path)}`);
        }
        prev = prev[prevKey];
    }
    return root[0];
}
exports.setValueAtKeyPath = setValueAtKeyPath;
/**
 * Creates the CXP extension API's {@link CXP#configuration} value.
 *
 * @param ext The CXP extension API handle.
 * @return The {@link CXP#configuration} value.
 */
function createExtConfiguration(ext, initial) {
    return new ExtConfiguration(ext, initial);
}
exports.createExtConfiguration = createExtConfiguration;
//# sourceMappingURL=configuration.js.map