"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
const cancel_1 = require("./cancel");
const cancel_2 = require("./cancel");
const events_1 = require("./events");
const linkedMap_1 = require("./linkedMap");
const messages_1 = require("./messages");
const trace_1 = require("./trace");
exports.NullLogger = Object.freeze({
    error: () => {
        /* noop */
    },
    warn: () => {
        /* noop */
    },
    info: () => {
        /* noop */
    },
    log: () => {
        /* noop */
    },
});
var ConnectionErrors;
(function (ConnectionErrors) {
    /**
     * The connection is closed.
     */
    ConnectionErrors[ConnectionErrors["Closed"] = 1] = "Closed";
    /**
     * The connection got unsubscribed (i.e., disposed).
     */
    ConnectionErrors[ConnectionErrors["Unsubscribed"] = 2] = "Unsubscribed";
    /**
     * The connection is already in listening mode.
     */
    ConnectionErrors[ConnectionErrors["AlreadyListening"] = 3] = "AlreadyListening";
})(ConnectionErrors = exports.ConnectionErrors || (exports.ConnectionErrors = {}));
class ConnectionError extends Error {
    constructor(code, message) {
        super(message);
        this.code = code;
        Object.setPrototypeOf(this, ConnectionError.prototype);
    }
}
exports.ConnectionError = ConnectionError;
function createMessageConnection(transports, logger, strategy) {
    if (!logger) {
        logger = exports.NullLogger;
    }
    return _createMessageConnection(transports, logger, strategy);
}
exports.createMessageConnection = createMessageConnection;
var ConnectionState;
(function (ConnectionState) {
    ConnectionState[ConnectionState["New"] = 1] = "New";
    ConnectionState[ConnectionState["Listening"] = 2] = "Listening";
    ConnectionState[ConnectionState["Closed"] = 3] = "Closed";
    ConnectionState[ConnectionState["Unsubscribed"] = 4] = "Unsubscribed";
})(ConnectionState || (ConnectionState = {}));
function _createMessageConnection(transports, logger, strategy) {
    let sequenceNumber = 0;
    let notificationSquenceNumber = 0;
    let unknownResponseSquenceNumber = 0;
    const version = '2.0';
    let starRequestHandler;
    const requestHandlers = Object.create(null);
    let starNotificationHandler;
    const notificationHandlers = Object.create(null);
    let timer;
    let messageQueue = new linkedMap_1.LinkedMap();
    let responsePromises = Object.create(null);
    let requestTokens = Object.create(null);
    let trace = trace_1.Trace.Off;
    let tracer = trace_1.noopTracer;
    let state = ConnectionState.New;
    const errorEmitter = new events_1.Emitter();
    const closeEmitter = new events_1.Emitter();
    const unhandledNotificationEmitter = new events_1.Emitter();
    const unsubscribeEmitter = new events_1.Emitter();
    function createRequestQueueKey(id) {
        return 'req-' + id.toString();
    }
    function createResponseQueueKey(id) {
        if (id === null) {
            return 'res-unknown-' + (++unknownResponseSquenceNumber).toString();
        }
        else {
            return 'res-' + id.toString();
        }
    }
    function createNotificationQueueKey() {
        return 'not-' + (++notificationSquenceNumber).toString();
    }
    function addMessageToQueue(queue, message) {
        if (messages_1.isRequestMessage(message)) {
            queue.set(createRequestQueueKey(message.id), message);
        }
        else if (messages_1.isResponseMessage(message)) {
            queue.set(createResponseQueueKey(message.id), message);
        }
        else {
            queue.set(createNotificationQueueKey(), message);
        }
    }
    function cancelUndispatched(_message) {
        return undefined;
    }
    function isListening() {
        return state === ConnectionState.Listening;
    }
    function isClosed() {
        return state === ConnectionState.Closed;
    }
    function isUnsubscribed() {
        return state === ConnectionState.Unsubscribed;
    }
    function closeHandler() {
        if (state === ConnectionState.New || state === ConnectionState.Listening) {
            state = ConnectionState.Closed;
            closeEmitter.fire(undefined);
        }
        // If the connection is unsubscribed don't sent close events.
    }
    function readErrorHandler(error) {
        errorEmitter.fire([error, undefined, undefined]);
    }
    function writeErrorHandler(data) {
        errorEmitter.fire(data);
    }
    transports.reader.onClose(closeHandler);
    transports.reader.onError(readErrorHandler);
    transports.writer.onClose(closeHandler);
    transports.writer.onError(writeErrorHandler);
    function triggerMessageQueue() {
        if (timer || messageQueue.size === 0) {
            return;
        }
        timer = setImmediateCompat(() => {
            timer = undefined;
            processMessageQueue();
        });
    }
    function processMessageQueue() {
        if (messageQueue.size === 0) {
            return;
        }
        const message = messageQueue.shift();
        try {
            if (messages_1.isRequestMessage(message)) {
                handleRequest(message);
            }
            else if (messages_1.isNotificationMessage(message)) {
                handleNotification(message);
            }
            else if (messages_1.isResponseMessage(message)) {
                handleResponse(message);
            }
            else {
                handleInvalidMessage(message);
            }
        }
        finally {
            triggerMessageQueue();
        }
    }
    const callback = message => {
        try {
            // We have received a cancellation message. Check if the message is still in the queue and cancel it if
            // allowed to do so.
            if (messages_1.isNotificationMessage(message) && message.method === cancel_1.CancelNotification.type.method) {
                const key = createRequestQueueKey(message.params.id);
                const toCancel = messageQueue.get(key);
                if (messages_1.isRequestMessage(toCancel)) {
                    const response = strategy && strategy.cancelUndispatched
                        ? strategy.cancelUndispatched(toCancel, cancelUndispatched)
                        : cancelUndispatched(toCancel);
                    if (response && (response.error !== undefined || response.result !== undefined)) {
                        messageQueue.delete(key);
                        response.id = toCancel.id;
                        tracer.responseCanceled(response, toCancel, message);
                        transports.writer.write(response);
                        return;
                    }
                }
            }
            addMessageToQueue(messageQueue, message);
        }
        finally {
            triggerMessageQueue();
        }
    };
    function handleRequest(requestMessage) {
        if (isUnsubscribed()) {
            // we return here silently since we fired an event when the
            // connection got unsubscribed.
            return;
        }
        const startTime = Date.now();
        function reply(resultOrError) {
            const message = {
                jsonrpc: version,
                id: requestMessage.id,
            };
            if (resultOrError instanceof messages_1.ResponseError) {
                message.error = resultOrError.toJson();
            }
            else {
                message.result = resultOrError === undefined ? null : resultOrError;
            }
            tracer.responseSent(message, requestMessage, startTime);
            transports.writer.write(message);
        }
        function replyError(error) {
            const message = {
                jsonrpc: version,
                id: requestMessage.id,
                error: error.toJson(),
            };
            tracer.responseSent(message, requestMessage, startTime);
            transports.writer.write(message);
        }
        function replySuccess(result) {
            // The JSON RPC defines that a response must either have a result or an error
            // So we can't treat undefined as a valid response result.
            if (result === undefined) {
                result = null;
            }
            const message = {
                jsonrpc: version,
                id: requestMessage.id,
                result,
            };
            tracer.responseSent(message, requestMessage, startTime);
            transports.writer.write(message);
        }
        tracer.requestReceived(requestMessage);
        const element = requestHandlers[requestMessage.method];
        const requestHandler = element && element.handler;
        if (requestHandler || starRequestHandler) {
            const cancellationSource = new cancel_2.CancellationTokenSource();
            const tokenKey = String(requestMessage.id);
            requestTokens[tokenKey] = cancellationSource;
            try {
                const params = requestMessage.params !== undefined ? requestMessage.params : null;
                const handlerResult = requestHandler
                    ? requestHandler(params, cancellationSource.token)
                    : starRequestHandler(requestMessage.method, params, cancellationSource.token);
                const promise = handlerResult;
                if (!handlerResult) {
                    delete requestTokens[tokenKey];
                    replySuccess(handlerResult);
                }
                else if (promise.then) {
                    promise.then((resultOrError) => {
                        delete requestTokens[tokenKey];
                        reply(resultOrError);
                    }, error => {
                        delete requestTokens[tokenKey];
                        if (error instanceof messages_1.ResponseError) {
                            replyError(error);
                        }
                        else if (error && typeof error.message === 'string') {
                            replyError(new messages_1.ResponseError(messages_1.ErrorCodes.InternalError, `Request ${requestMessage.method} failed with message: ${error.message}`));
                        }
                        else {
                            replyError(new messages_1.ResponseError(messages_1.ErrorCodes.InternalError, `Request ${requestMessage.method} failed unexpectedly without providing any details.`));
                        }
                    });
                }
                else {
                    delete requestTokens[tokenKey];
                    reply(handlerResult);
                }
            }
            catch (error) {
                delete requestTokens[tokenKey];
                if (error instanceof messages_1.ResponseError) {
                    reply(error);
                }
                else if (error && typeof error.message === 'string') {
                    replyError(new messages_1.ResponseError(messages_1.ErrorCodes.InternalError, `Request ${requestMessage.method} failed with message: ${error.message}`));
                }
                else {
                    replyError(new messages_1.ResponseError(messages_1.ErrorCodes.InternalError, `Request ${requestMessage.method} failed unexpectedly without providing any details.`));
                }
            }
        }
        else {
            replyError(new messages_1.ResponseError(messages_1.ErrorCodes.MethodNotFound, `Unhandled method ${requestMessage.method}`));
        }
    }
    function handleResponse(responseMessage) {
        if (isUnsubscribed()) {
            // See handle request.
            return;
        }
        if (responseMessage.id === null) {
            if (responseMessage.error) {
                logger.error(`Received response message without id: Error is: \n${JSON.stringify(responseMessage.error, undefined, 4)}`);
            }
            else {
                logger.error(`Received response message without id. No further error information provided.`);
            }
        }
        else {
            const key = String(responseMessage.id);
            const responsePromise = responsePromises[key];
            if (responsePromise) {
                tracer.responseReceived(responseMessage, responsePromise.request || responsePromise.method, responsePromise.timerStart);
                delete responsePromises[key];
                try {
                    if (responseMessage.error) {
                        const error = responseMessage.error;
                        responsePromise.reject(new messages_1.ResponseError(error.code, error.message, error.data));
                    }
                    else if (responseMessage.result !== undefined) {
                        responsePromise.resolve(responseMessage.result);
                    }
                    else {
                        throw new Error('Should never happen.');
                    }
                }
                catch (error) {
                    if (error.message) {
                        logger.error(`Response handler '${responsePromise.method}' failed with message: ${error.message}`);
                    }
                    else {
                        logger.error(`Response handler '${responsePromise.method}' failed unexpectedly.`);
                    }
                }
            }
            else {
                tracer.unknownResponseReceived(responseMessage);
            }
        }
    }
    function handleNotification(message) {
        if (isUnsubscribed()) {
            // See handle request.
            return;
        }
        let notificationHandler;
        if (message.method === cancel_1.CancelNotification.type.method) {
            notificationHandler = (params) => {
                const id = params.id;
                const source = requestTokens[String(id)];
                if (source) {
                    source.cancel();
                }
            };
        }
        else {
            const element = notificationHandlers[message.method];
            if (element) {
                notificationHandler = element.handler;
            }
        }
        if (notificationHandler || starNotificationHandler) {
            try {
                tracer.notificationReceived(message);
                notificationHandler
                    ? notificationHandler(message.params)
                    : starNotificationHandler(message.method, message.params);
            }
            catch (error) {
                if (error.message) {
                    logger.error(`Notification handler '${message.method}' failed with message: ${error.message}`);
                }
                else {
                    logger.error(`Notification handler '${message.method}' failed unexpectedly.`);
                }
            }
        }
        else {
            unhandledNotificationEmitter.fire(message);
        }
    }
    function handleInvalidMessage(message) {
        if (!message) {
            logger.error('Received empty message.');
            return;
        }
        logger.error(`Received message which is neither a response nor a notification message:\n${JSON.stringify(message, null, 4)}`);
        // Test whether we find an id to reject the promise
        const responseMessage = message;
        if (typeof responseMessage.id === 'string' || typeof responseMessage.id === 'number') {
            const key = String(responseMessage.id);
            const responseHandler = responsePromises[key];
            if (responseHandler) {
                responseHandler.reject(new Error('The received response has neither a result nor an error property.'));
            }
        }
    }
    function throwIfClosedOrUnsubscribed() {
        if (isClosed()) {
            throw new ConnectionError(ConnectionErrors.Closed, 'Connection is closed.');
        }
        if (isUnsubscribed()) {
            throw new ConnectionError(ConnectionErrors.Unsubscribed, 'Connection is unsubscribed.');
        }
    }
    function throwIfListening() {
        if (isListening()) {
            throw new ConnectionError(ConnectionErrors.AlreadyListening, 'Connection is already listening');
        }
    }
    function throwIfNotListening() {
        if (!isListening()) {
            throw new Error('Call listen() first.');
        }
    }
    const connection = {
        sendNotification: (type, params) => {
            throwIfClosedOrUnsubscribed();
            let method;
            if (typeof type === 'string') {
                method = type;
            }
            else {
                method = type.method;
            }
            const notificationMessage = {
                jsonrpc: version,
                method,
                params,
            };
            tracer.notificationSent(notificationMessage);
            transports.writer.write(notificationMessage);
        },
        onNotification: (type, handler) => {
            throwIfClosedOrUnsubscribed();
            if (util_1.isFunction(type)) {
                starNotificationHandler = type;
            }
            else if (handler) {
                if (typeof type === 'string') {
                    notificationHandlers[type] = { type: undefined, handler };
                }
                else {
                    notificationHandlers[type.method] = { type, handler };
                }
            }
        },
        sendRequest: (type, params, token) => {
            throwIfClosedOrUnsubscribed();
            throwIfNotListening();
            let method;
            if (typeof type === 'string') {
                method = type;
            }
            else {
                method = type.method;
                token = cancel_2.CancellationToken.is(token) ? token : undefined;
            }
            const id = sequenceNumber++;
            const result = new Promise((resolve, reject) => {
                const requestMessage = {
                    jsonrpc: version,
                    id,
                    method,
                    params,
                };
                let responsePromise = {
                    method,
                    request: trace === trace_1.Trace.Verbose ? requestMessage : undefined,
                    timerStart: Date.now(),
                    resolve,
                    reject,
                };
                tracer.requestSent(requestMessage);
                try {
                    transports.writer.write(requestMessage);
                }
                catch (e) {
                    // Writing the message failed. So we need to reject the promise.
                    responsePromise.reject(new messages_1.ResponseError(messages_1.ErrorCodes.MessageWriteError, e.message ? e.message : 'Unknown reason'));
                    responsePromise = null;
                }
                if (responsePromise) {
                    responsePromises[String(id)] = responsePromise;
                }
            });
            if (token) {
                token.onCancellationRequested(() => {
                    connection.sendNotification(cancel_1.CancelNotification.type, { id });
                });
            }
            return result;
        },
        onRequest: (type, handler) => {
            throwIfClosedOrUnsubscribed();
            if (util_1.isFunction(type)) {
                starRequestHandler = type;
            }
            else if (handler) {
                if (typeof type === 'string') {
                    requestHandlers[type] = { type: undefined, handler };
                }
                else {
                    requestHandlers[type.method] = { type, handler };
                }
            }
        },
        trace: (_value, _tracer, sendNotification = false) => {
            trace = _value;
            if (trace === trace_1.Trace.Off) {
                tracer = trace_1.noopTracer;
            }
            else {
                tracer = _tracer;
            }
            if (sendNotification && !isClosed() && !isUnsubscribed()) {
                connection.sendNotification(trace_1.SetTraceNotification.type, { value: trace_1.Trace.toString(_value) });
            }
        },
        onError: errorEmitter.event,
        onClose: closeEmitter.event,
        onUnhandledNotification: unhandledNotificationEmitter.event,
        onUnsubscribe: unsubscribeEmitter.event,
        unsubscribe: () => {
            if (isUnsubscribed()) {
                return;
            }
            state = ConnectionState.Unsubscribed;
            unsubscribeEmitter.fire(undefined);
            const error = new Error('Connection got unsubscribed.');
            for (const key of Object.keys(responsePromises)) {
                responsePromises[key].reject(error);
            }
            responsePromises = Object.create(null);
            requestTokens = Object.create(null);
            messageQueue = new linkedMap_1.LinkedMap();
            // Test for backwards compatibility
            // tslint:disable-next-line:no-unbound-method
            if (util_1.isFunction(transports.writer.unsubscribe)) {
                transports.writer.unsubscribe();
            }
            // tslint:disable-next-line:no-unbound-method
            if (util_1.isFunction(transports.reader.unsubscribe)) {
                transports.reader.unsubscribe();
            }
        },
        listen: () => {
            throwIfClosedOrUnsubscribed();
            throwIfListening();
            state = ConnectionState.Listening;
            transports.reader.listen(callback);
        },
    };
    connection.onNotification(trace_1.LogTraceNotification.type, params => {
        tracer.log(params.message, trace === trace_1.Trace.Verbose ? params.verbose : undefined);
    });
    return connection;
}
/** Support browser and node environments without needing a transpiler. */
function setImmediateCompat(f) {
    if (typeof setImmediate !== 'undefined') {
        return setImmediate(f);
    }
    return setTimeout(f, 0);
}
//# sourceMappingURL=connection.js.map