import { Observable, Unsubscribable } from 'rxjs';
import { MessageTransports } from '../jsonrpc2/connection';
import { GenericNotificationHandler, GenericRequestHandler, NotificationHandler, RequestHandler } from '../jsonrpc2/handlers';
import { NotificationType, RequestType } from '../jsonrpc2/messages';
import { Trace, Tracer } from '../jsonrpc2/trace';
import { InitializeResult, RegistrationParams, UnregistrationParams } from '../protocol';
import { DocumentSelector } from '../types/document';
import { URI } from '../types/textDocument';
import { ErrorHandler, InitializationFailedHandler } from './errorHandler';
import { DynamicFeature, StaticFeature } from './features/common';
import { Middleware } from './middleware';
/** Options for creating a new client. */
export interface ClientOptions {
    root: URI | null;
    documentSelector?: DocumentSelector;
    initializationOptions?: any | (() => any);
    /** Called when initialization fails to determine how to proceed. */
    initializationFailedHandler?: InitializationFailedHandler;
    /** Called when an error or close occurs to determine how to proceed. */
    errorHandler?: ErrorHandler;
    middleware?: Readonly<Middleware>;
    /** Called to create the connection to the server. */
    createMessageTransports: () => MessageTransports | Promise<MessageTransports>;
    /** Trace log level. The tracer must also be set. */
    trace?: Trace;
    /** Logs messages sent to and received from the server, and trace log messages sent from the server. */
    tracer?: Tracer;
    /**
     * Experimental capabilities implemented by the client (that are not defined
     * by the CXP specification). These capabilities are passed verbatim to
     * extensions in the initialize request's capabilities.experimental
     * property.
     */
    experimentalClientCapabilities?: any;
}
/** The client options, after defaults have been set that make certain fields required. */
interface ResolvedClientOptions extends Pick<ClientOptions, Exclude<keyof ClientOptions, 'trace'>> {
    initializationFailedHandler: InitializationFailedHandler;
    errorHandler: ErrorHandler;
    middleware: Readonly<Middleware>;
    tracer: Tracer;
    experimentalClientCapabilities: any;
}
/** The possible states of a client. */
export declare enum ClientState {
    /** The initial state of the client. It has never been activated. */
    Initial = 0,
    /** The client is establishing the connection to the server and sending the "initialize" message. */
    Connecting = 1,
    /**
     * The connection is established and the client is waiting for and handling the server's "initialize" result.
     */
    Initializing = 2,
    /** The client encountered an error while activating. */
    ActivateFailed = 3,
    /** The client has finished initialization and is in operation. */
    Active = 4,
    /** The client is gracefully shutting down the connection. */
    ShuttingDown = 5,
    /** The client is deactivated (after having previously been activated). */
    Stopped = 6
}
/**
 * The client communicates with a CXP server.
 */
export declare class Client implements Unsubscribable {
    readonly id: string;
    readonly name: string;
    readonly options: Readonly<ResolvedClientOptions>;
    private _initializeResult;
    readonly initializeResult: InitializeResult | null;
    private _state;
    readonly state: Observable<ClientState>;
    private connectionPromise;
    private connection;
    private onStop;
    private _trace;
    constructor(id: string, name: string, { trace, ...options }: ClientOptions);
    private readonly isConnectionActive;
    needsStop(): boolean;
    /**
     * Activates the client, which causes it to start connecting (and to reestablish the connection when it drops,
     * as directed by the initializationFailedHandler).
     *
     * To watch client state, use Client#state. To log client errors, provide an initializationFailedHandler and
     * errorHandler in ClientOptions.
     */
    activate(): void;
    /**
     * Activates the client and returns a promise that resolves when the initial activation finishes (or fails and
     * no retry will occur). Used by tests.
     */
    protected activateAndWait(): Promise<void>;
    private resolveConnection;
    private initialize;
    protected handleConnectionClosed(): void;
    private handleConnectionError;
    /**
     * Stops the client, which causes it to gracefully shut down the current connection (if any) and remain
     * disconnected until a subsequent call to Client#activate.
     *
     * @returns a promise that resolves when shutdown completes, or immediately if the client is not connected
     */
    stop(): Promise<void>;
    private stopAtState;
    private unregisterFeatures;
    sendRequest<P, R, E, RO>(type: RequestType<P, R, E, RO>, params: P): Promise<R>;
    sendRequest<R>(method: string, params?: any): Promise<R>;
    onRequest<P, R, E, RO>(type: RequestType<P, R, E, RO>, handler: RequestHandler<P, R, E>): void;
    onRequest<R, E>(method: string, handler: GenericRequestHandler<R, E>): void;
    sendNotification<P, RO>(type: NotificationType<P, RO>, params?: P): void;
    sendNotification(method: string, params?: any): void;
    onNotification<P, RO>(type: NotificationType<P, RO>, handler: NotificationHandler<P>): void;
    onNotification(method: string, handler: GenericNotificationHandler): void;
    trace: Trace;
    protected readonly features: (StaticFeature | DynamicFeature<any>)[];
    private readonly _method2Message;
    private readonly _dynamicFeatures;
    registerFeature(feature: StaticFeature | DynamicFeature<any>): void;
    protected handleRegistrationRequest(params: RegistrationParams): void;
    protected handleUnregistrationRequest(params: UnregistrationParams): void;
    unsubscribe(): void;
}
export {};
