var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import { basename } from 'path';
import { BehaviorSubject } from 'rxjs';
import { noopTracer, Trace } from '../jsonrpc2/trace';
import { InitializedNotification, RegistrationRequest, UnregistrationRequest, } from '../protocol';
import { isFunction, tryCatchPromise } from '../util';
import { createConnection } from './connection';
import { CloseAction, DefaultErrorHandler, ErrorAction, } from './errorHandler';
import { DynamicFeature } from './features/common';
/** The possible states of a client. */
export var ClientState;
(function (ClientState) {
    /** The initial state of the client. It has never been activated. */
    ClientState[ClientState["Initial"] = 0] = "Initial";
    /** The client is establishing the connection to the server and sending the "initialize" message. */
    ClientState[ClientState["Connecting"] = 1] = "Connecting";
    /**
     * The connection is established and the client is waiting for and handling the server's "initialize" result.
     */
    ClientState[ClientState["Initializing"] = 2] = "Initializing";
    /** The client encountered an error while activating. */
    ClientState[ClientState["ActivateFailed"] = 3] = "ActivateFailed";
    /** The client has finished initialization and is in operation. */
    ClientState[ClientState["Active"] = 4] = "Active";
    /** The client is gracefully shutting down the connection. */
    ClientState[ClientState["ShuttingDown"] = 5] = "ShuttingDown";
    /** The client is deactivated (after having previously been activated). */
    ClientState[ClientState["Stopped"] = 6] = "Stopped";
})(ClientState || (ClientState = {}));
/**
 * The client communicates with a CXP server.
 */
export class Client {
    constructor(id, name, _a) {
        var { trace } = _a, options = __rest(_a, ["trace"]);
        this.id = id;
        this.name = name;
        this._initializeResult = null;
        this._state = new BehaviorSubject(ClientState.Initial);
        this.connectionPromise = null;
        this.connection = null;
        this.onStop = null;
        this.features = [];
        this._method2Message = new Map();
        this._dynamicFeatures = new Map();
        this.options = Object.assign({}, options, { initializationFailedHandler: options.initializationFailedHandler || (() => Promise.resolve(false)), errorHandler: options.errorHandler || new DefaultErrorHandler(), middleware: options.middleware || {}, tracer: options.tracer || noopTracer, experimentalClientCapabilities: options.experimentalClientCapabilities || {} });
        this._trace = trace || Trace.Off;
    }
    get initializeResult() {
        return this._initializeResult;
    }
    get state() {
        return this._state;
    }
    get isConnectionActive() {
        return ((this._state.value === ClientState.Initializing || this._state.value === ClientState.Active) &&
            this.connection !== null);
    }
    needsStop() {
        return (this._state.value === ClientState.Connecting ||
            this._state.value === ClientState.Initializing ||
            this._state.value === ClientState.Active);
    }
    /**
     * Activates the client, which causes it to start connecting (and to reestablish the connection when it drops,
     * as directed by the initializationFailedHandler).
     *
     * To watch client state, use Client#state. To log client errors, provide an initializationFailedHandler and
     * errorHandler in ClientOptions.
     */
    activate() {
        // Callers should subscribe to Client#state instead of awaiting the activation.
        this.activateAndWait().then(null, () => void 0);
    }
    /**
     * Activates the client and returns a promise that resolves when the initial activation finishes (or fails and
     * no retry will occur). Used by tests.
     */
    activateAndWait() {
        this._state.next(ClientState.Connecting);
        let activateConnection = null; // track so we know if we're dealing with the same value upon error
        return this.resolveConnection()
            .then(connection => {
            activateConnection = connection;
            connection.listen();
            return this.initialize(connection);
        })
            .then(null, err => {
            console.error(err);
            // Only update state if it pertains to the same connection we started with.
            if (activateConnection === this.connection && this._state.value !== ClientState.Stopped) {
                this._state.next(ClientState.ActivateFailed);
            }
        });
    }
    resolveConnection() {
        if (!this.connectionPromise) {
            this.connectionPromise = tryCatchPromise(this.options.createMessageTransports).then(transports => createConnection(transports, (error, message, count) => this.handleConnectionError(error, message, count), () => this.handleConnectionClosed()));
        }
        return this.connectionPromise;
    }
    initialize(connection) {
        connection.trace(this._trace, this.options.tracer);
        const initParams = {
            root: this.options.root,
            capabilities: {
                experimental: this.options.experimentalClientCapabilities,
            },
            configurationCascade: { merged: {} },
            initializationOptions: isFunction(this.options.initializationOptions)
                ? this.options.initializationOptions()
                : this.options.initializationOptions,
            workspaceFolders: this.options.root
                ? [{ name: basename(this.options.root), uri: this.options.root }]
                : null,
            trace: Trace.toString(this._trace),
        };
        // Fill initialize params and client capabilities from features.
        for (const feature of this.features) {
            if (isFunction(feature.fillClientCapabilities)) {
                feature.fillClientCapabilities(initParams.capabilities);
            }
            if (isFunction(feature.fillInitializeParams)) {
                feature.fillInitializeParams(initParams);
            }
        }
        return connection
            .initialize(initParams)
            .then(result => {
            // If this client was stopped during the initialize call, don't continue'
            if (this._state.value === ClientState.ShuttingDown || this._state.value === ClientState.Stopped) {
                return;
            }
            this.connection = connection;
            this._initializeResult = result;
            this._state.next(ClientState.Initializing);
            connection.onRequest(RegistrationRequest.type, params => this.handleRegistrationRequest(params));
            connection.onRequest(UnregistrationRequest.type, params => this.handleUnregistrationRequest(params));
            connection.sendNotification(InitializedNotification.type, {});
            // Initialize features.
            for (const feature of this.features) {
                feature.initialize(result.capabilities, this.options.documentSelector);
            }
            this._state.next(ClientState.Active);
        })
            .then(null, err => Promise.resolve(this.options.initializationFailedHandler(err)).then(reinitialize => {
            if (reinitialize) {
                return this.initialize(connection);
            }
            if (connection) {
                connection.unsubscribe();
            }
            return this.stopAtState(ClientState.ActivateFailed);
        }));
    }
    handleConnectionClosed() {
        // Check whether this is a normal shutdown in progress or the client stopped normally.
        if (this._state.value === ClientState.ShuttingDown ||
            this._state.value === ClientState.Stopped ||
            this._state.value === ClientState.ActivateFailed) {
            return;
        }
        try {
            if (this.connection) {
                this.connection.unsubscribe();
            }
        }
        catch (error) {
            // Unsubscribing a connection could fail if error cases.
        }
        this.connectionPromise = null;
        this.connection = null;
        this.unregisterFeatures();
        let action = Promise.resolve(CloseAction.DoNotReconnect);
        try {
            action = Promise.resolve(this.options.errorHandler.closed());
        }
        catch (error) {
            // Ignore sync errors from the error handler.
        }
        // tslint:disable-next-line:no-floating-promises
        action
            .catch(() => CloseAction.DoNotReconnect) // ignore async errors from the error handler
            .then(action => {
            if (action === CloseAction.DoNotReconnect) {
                this._state.next(ClientState.Stopped);
            }
            else if (action === CloseAction.Reconnect) {
                this._state.next(ClientState.Initial);
                this.activate();
            }
        });
    }
    handleConnectionError(error, message, count) {
        const action = this.options.errorHandler.error(error, message, count);
        if (action === ErrorAction.ShutDown) {
            this.stopAtState(this.isConnectionActive ? ClientState.Stopped : ClientState.ActivateFailed).then(null, err => console.error(err));
        }
    }
    /**
     * Stops the client, which causes it to gracefully shut down the current connection (if any) and remain
     * disconnected until a subsequent call to Client#activate.
     *
     * @returns a promise that resolves when shutdown completes, or immediately if the client is not connected
     */
    stop() {
        return this.stopAtState(ClientState.Stopped);
    }
    stopAtState(endState) {
        this._initializeResult = null;
        if (!this.connectionPromise) {
            this._state.next(ClientState.Stopped);
            return Promise.resolve();
        }
        if (this._state.value === ClientState.ShuttingDown && this.onStop) {
            return this.onStop;
        }
        const closeConnection = (connection) => {
            // It's possible for the connection to be alive and this.isConnectionActive === false (e.g., if we're
            // still waiting to hear back from the server), so make sure we close it.
            if (connection) {
                connection.unsubscribe();
            }
            if (connection !== this.connection) {
                // Another connection was created while we were preparing this one to close. Don't modify any state
                // because the state reflects the new connection now.
                return;
            }
            if (this._state.value !== endState) {
                this._state.next(endState);
            }
            this.onStop = null;
            this.connectionPromise = null;
            this.connection = null;
        };
        // If we are connected to a server, then shut down gracefully. Otherwise (if we aren't connected to a
        // server, including if the connection never succeeded), just close the connection (if any) immediately.
        const wasConnectionActive = this.isConnectionActive;
        if (this.isConnectionActive) {
            this._state.next(ClientState.ShuttingDown);
        }
        else {
            this._state.next(endState);
        }
        this.unregisterFeatures();
        if (wasConnectionActive) {
            // Shut down gracefully before closing the connection.
            const connection = this.connection;
            return (this.onStop = connection
                .shutdown()
                .catch(() => {
                /* Ignore shutdown errors from server. */
            })
                .then(() => {
                connection.exit();
                closeConnection(connection);
            }));
        }
        // Otherwise, just close the connection.
        closeConnection(this.connection);
        return Promise.resolve();
    }
    unregisterFeatures() {
        for (const feature of this.features) {
            if (DynamicFeature.is(feature)) {
                feature.unregisterAll();
            }
            else if (feature.deinitialize) {
                feature.deinitialize();
            }
        }
    }
    sendRequest(type, ...params) {
        if (!this.isConnectionActive) {
            throw new Error('connection is inactive');
        }
        return Promise.resolve(this.connection.sendRequest(type, ...params));
    }
    onRequest(type, handler) {
        if (!this.isConnectionActive) {
            throw new Error('connection is inactive');
        }
        this.connection.onRequest(type, handler);
    }
    sendNotification(type, params) {
        if (!this.isConnectionActive) {
            throw new Error('connection is inactive');
        }
        this.connection.sendNotification(type, params);
    }
    onNotification(type, handler) {
        if (!this.isConnectionActive) {
            throw new Error('connection is inactive');
        }
        this.connection.onNotification(type, handler);
    }
    get trace() {
        return this._trace;
    }
    set trace(value) {
        this._trace = value;
        if (this.connection) {
            this.connection.trace(value, this.options.tracer);
        }
    }
    registerFeature(feature) {
        if (DynamicFeature.is(feature)) {
            const messages = Array.isArray(feature.messages) ? feature.messages : [feature.messages];
            for (const message of messages) {
                if (this._method2Message.has(message.method)) {
                    throw new Error(`dynamic feature is already registered for method ${JSON.stringify(message.method)}`);
                }
                this._method2Message.set(message.method, message);
                this._dynamicFeatures.set(message.method, feature);
            }
        }
        this.features.push(feature);
    }
    handleRegistrationRequest(params) {
        for (const registration of params.registrations) {
            const feature = this._dynamicFeatures.get(registration.method);
            if (!feature) {
                throw new Error(`dynamic feature not found: ${JSON.stringify(registration.method)}`);
            }
            const options = registration.registerOptions || {};
            if (!options.documentSelector && this.options.documentSelector) {
                options.documentSelector = this.options.documentSelector;
            }
            const data = {
                id: registration.id,
                registerOptions: options,
                overwriteExisting: registration.overwriteExisting,
            };
            feature.register(this._method2Message.get(registration.method), data);
        }
    }
    handleUnregistrationRequest(params) {
        for (const unregistration of params.unregisterations) {
            const feature = this._dynamicFeatures.get(unregistration.method);
            if (!feature) {
                throw new Error(`dynamic feature not found: ${JSON.stringify(unregistration.method)}`);
            }
            feature.unregister(unregistration.id);
        }
    }
    unsubscribe() {
        // Immediately unregister feature registrations, even before the connection shuts down, since calling
        // unsubscribe is evidence that the consumer doesn't want this client's data anymore.
        this.unregisterFeatures();
        if (this.needsStop()) {
            this.stop().then(null, err => console.error(err));
        }
    }
}
//# sourceMappingURL=client.js.map