import { Subscription } from 'rxjs';
import uuidv4 from 'uuid/v4';
import { ExecuteCommandRequest, } from '../../protocol';
import { ensure } from './common';
/**
 * Support for the commands executed on the server by the client (workspace/executeCommand requests to the server).
 */
export class ExecuteCommandFeature {
    constructor(client, registry) {
        this.client = client;
        this.registry = registry;
        this.commands = new Map();
    }
    get messages() {
        return ExecuteCommandRequest.type;
    }
    fillClientCapabilities(capabilities) {
        ensure(ensure(capabilities, 'workspace'), 'executeCommand').dynamicRegistration = true;
    }
    initialize(capabilities) {
        if (!capabilities.executeCommandProvider) {
            return;
        }
        this.register(this.messages, {
            id: uuidv4(),
            registerOptions: Object.assign({}, capabilities.executeCommandProvider),
        });
    }
    register(_message, data) {
        const existing = this.commands.has(data.id);
        if (existing) {
            throw new Error(`registration already exists with ID ${data.id}`);
        }
        const sub = new Subscription();
        for (const command of data.registerOptions.commands) {
            try {
                sub.add(this.registry.registerCommand({
                    command,
                    run: (...args) => this.client.sendRequest(ExecuteCommandRequest.type, {
                        command,
                        arguments: args,
                    }),
                }));
            }
            catch (err) {
                // Unregister commands that were successfully registered if any of them fails, so that this invocation
                // is atomic (all-or-none).
                sub.unsubscribe();
                throw err;
            }
        }
        this.commands.set(data.id, sub);
    }
    unregister(id) {
        const sub = this.commands.get(id);
        if (!sub) {
            throw new Error(`no registration with ID ${id}`);
        }
        if (sub) {
            sub.unsubscribe();
        }
        this.commands.delete(id);
    }
    unregisterAll() {
        for (const sub of this.commands.values()) {
            sub.unsubscribe();
        }
        this.commands.clear();
    }
}
//# sourceMappingURL=command.js.map