import { Unsubscribable } from 'rxjs';
import { MessageType as RPCMessageType } from '../../jsonrpc2/messages';
import { ClientCapabilities, InitializeParams, ServerCapabilities } from '../../protocol';
import { DocumentSelector } from '../../types/document';
import { Client } from '../client';
/** A client feature that exposes functionality that is always enabled. */
export interface StaticFeature {
    fillInitializeParams?: (params: InitializeParams) => void;
    fillClientCapabilities?: (capabilities: ClientCapabilities) => void;
    initialize(capabilities: ServerCapabilities, documentSelector: DocumentSelector | undefined): void;
    /** Free resources acquired in initialize. */
    deinitialize?: () => void;
}
/** Common arguments used when registering a dynamic feature. */
export interface RegistrationData<T> {
    id: string;
    registerOptions: T;
    overwriteExisting?: boolean;
}
/** A client feature that exposes functionality that the server can enable, configure, and disable. */
export interface DynamicFeature<T> {
    messages: RPCMessageType | RPCMessageType[];
    fillInitializeParams?: (params: InitializeParams) => void;
    fillClientCapabilities(capabilities: ClientCapabilities): void;
    initialize(capabilities: ServerCapabilities, documentSelector: DocumentSelector | undefined): void;
    register(message: RPCMessageType, data: RegistrationData<T>): void;
    unregister(id: string): void;
    /**
     * Unregisters all static and dynamic registrations and prepares the feature to be reused for a new CXP
     * connection.
     */
    unregisterAll(): void;
}
export declare namespace DynamicFeature {
    /** Reports whether the value is a DynamicFeature. */
    function is<T>(value: any): value is DynamicFeature<T>;
}
/** Common base class for client features. */
export declare abstract class Feature<O> implements DynamicFeature<O> {
    protected client: Client;
    private subscriptionsByID;
    constructor(client: Client);
    abstract readonly messages: RPCMessageType;
    register(message: RPCMessageType, data: RegistrationData<O>): void;
    protected abstract registerProvider(options: O): Unsubscribable;
    abstract fillClientCapabilities(capabilities: ClientCapabilities): void;
    abstract initialize(capabilities: ServerCapabilities, documentSelector: DocumentSelector): void;
    unregister(id: string): void;
    unregisterAll(): void;
}
export declare function ensure<T, K extends keyof T>(target: T, key: K): T[K];
