import uuidv4 from 'uuid/v4';
import { ensure } from './common';
/**
 * Support for user-facing features contributed by the server for use and display in the client's application.
 */
export class ContributionFeature {
    constructor(registry) {
        this.registry = registry;
        this.contributions = new Map();
    }
    get messages() {
        // This method is not actually a protocol method, but some value is required here because the client tracks
        // DynamicFeatures based on their method name.
        return { method: 'window/contribution' };
    }
    fillClientCapabilities(capabilities) {
        ensure(ensure(capabilities, 'window'), 'contribution').dynamicRegistration = true;
    }
    initialize(capabilities) {
        if (!capabilities.contributions) {
            return;
        }
        this.register(this.messages, {
            id: uuidv4(),
            registerOptions: capabilities.contributions,
        });
    }
    register(_message, data) {
        const existing = this.contributions.get(data.id);
        if (existing && !data.overwriteExisting) {
            throw new Error(`registration already exists with ID ${data.id}`);
        }
        const entry = { contributions: data.registerOptions };
        let subscription;
        if (data.overwriteExisting) {
            if (!existing) {
                throw new Error(`no existing registration to overwrite with ID ${data.id}`);
            }
            subscription = this.registry.replaceContributions(existing, entry);
        }
        else {
            subscription = this.registry.registerContributions(entry);
        }
        this.contributions.set(data.id, subscription);
    }
    unregister(id) {
        const sub = this.contributions.get(id);
        if (!sub) {
            throw new Error(`no registration with ID ${id}`);
        }
        sub.unsubscribe();
        this.contributions.delete(id);
    }
    unregisterAll() {
        for (const sub of this.contributions.values()) {
            sub.unsubscribe();
        }
        this.contributions.clear();
    }
}
//# sourceMappingURL=contribution.js.map