import uuidv4 from 'uuid/v4';
import { DefinitionRequest, ImplementationRequest, ReferencesRequest, TypeDefinitionRequest, } from '../../protocol';
import { ensure, Feature } from './common';
/**
 * Support for requests that retrieve a list of locations (e.g., textDocument/definition,
 * textDocument/implementation, and textDocument/typeDefinition).
 */
export class TextDocumentLocationFeature extends Feature {
    constructor(client, registry) {
        super(client);
        this.registry = registry;
    }
    initialize(capabilities, documentSelector) {
        if (!this.isSupported(capabilities) || !documentSelector) {
            return;
        }
        this.register(this.messages, {
            id: uuidv4(),
            registerOptions: { documentSelector },
        });
    }
    registerProvider(options) {
        const client = this.client;
        const provideTextDocumentLocation = params => client.sendRequest(this.messages, params);
        const middleware = this.getMiddleware ? this.getMiddleware(client.options.middleware) : undefined;
        return this.registry.registerProvider(options, (params) => middleware ? middleware(params, provideTextDocumentLocation) : provideTextDocumentLocation(params));
    }
}
/**
 * Support for definition requests (textDocument/definition requests to the server).
 */
export class TextDocumentDefinitionFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = DefinitionRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = ensure(ensure(capabilities, 'textDocument'), 'definition');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.definitionProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentDefinition;
    }
}
/**
 * Support for implementation requests (textDocument/implementation requests to the server).
 */
export class TextDocumentImplementationFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = ImplementationRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = ensure(ensure(capabilities, 'textDocument'), 'implementation');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.implementationProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentImplementation;
    }
}
/**
 * Support for type definition requests (textDocument/typeDefinition requests to the server).
 */
export class TextDocumentTypeDefinitionFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = TypeDefinitionRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = ensure(ensure(capabilities, 'textDocument'), 'typeDefinition');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.typeDefinitionProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentTypeDefinition;
    }
}
/**
 * Support for references requests (textDocument/references requests to the server).
 */
export class TextDocumentReferencesFeature extends TextDocumentLocationFeature {
    constructor() {
        super(...arguments);
        this.messages = ReferencesRequest.type;
    }
    fillClientCapabilities(capabilities) {
        const capability = ensure(ensure(capabilities, 'textDocument'), 'references');
        capability.dynamicRegistration = true;
    }
    isSupported(capabilities) {
        return !!capabilities.referencesProvider;
    }
    getMiddleware(middleware) {
        return middleware.provideTextDocumentReferences;
    }
}
//# sourceMappingURL=location.js.map