import { bufferCount, filter, map } from 'rxjs/operators';
import uuidv4 from 'uuid/v4';
import { DidCloseTextDocumentNotification, DidOpenTextDocumentNotification, } from '../../protocol';
import { match } from '../../types/textDocument';
import { ensure } from './common';
export class TextDocumentNotificationFeature {
    constructor(client, observable, type, middleware, createParams, selectorFilter) {
        this.client = client;
        this.observable = observable;
        this.type = type;
        this.middleware = middleware;
        this.createParams = createParams;
        this.selectorFilter = selectorFilter;
        this.subscription = null;
        this.selectors = new Map();
    }
    register(_message, data) {
        if (!data.registerOptions.documentSelector) {
            return;
        }
        if (this.selectors.has(data.id)) {
            throw new Error(`registration already exists with ID ${data.id}`);
        }
        this.selectors.set(data.id, data.registerOptions.documentSelector);
        if (!this.subscription) {
            this.subscription = this.observable.subscribe(data => this.callback(data));
        }
    }
    callback(data) {
        if (!this.selectorFilter || this.selectorFilter(this.selectors.values(), data)) {
            if (this.middleware) {
                this.middleware(data, data => this.client.sendNotification(this.type, this.createParams(data)));
            }
            else {
                this.client.sendNotification(this.type, this.createParams(data));
            }
            this.notificationSent(data);
        }
    }
    notificationSent(_data) {
        /* noop */
    }
    unregister(id) {
        if (!this.selectors.delete(id)) {
            throw new Error(`no registration with ID ${id}`);
        }
        this.selectors.delete(id);
        if (this.selectors.size === 0 && this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    }
    unregisterAll() {
        this.selectors.clear();
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    }
}
/**
 * Support for notifying the server of when the client opened a text document (textDocument/didOpen notifications
 * sent by the client to the server).
 */
export class TextDocumentDidOpenFeature extends TextDocumentNotificationFeature {
    constructor(client, environmentTextDocument) {
        super(client, environmentTextDocument.pipe(filter((v) => v !== null)), DidOpenTextDocumentNotification.type, client.options.middleware.didOpen, textDocument => ({
            textDocument,
        }), match);
    }
    get messages() {
        return DidOpenTextDocumentNotification.type;
    }
    fillClientCapabilities(capabilities) {
        ensure(ensure(capabilities, 'textDocument'), 'synchronization').dynamicRegistration = true;
    }
    initialize(capabilities, documentSelector) {
        const textDocumentSyncOptions = resolveTextDocumentSync(capabilities.textDocumentSync);
        if (documentSelector && textDocumentSyncOptions && textDocumentSyncOptions.openClose) {
            this.register(this.messages, {
                id: uuidv4(),
                registerOptions: { documentSelector },
            });
        }
    }
}
/**
 * Support for notifying the server of when the client closed a text document (textDocument/didClose notifications
 * sent by the client to the server).
 */
export class TextDocumentDidCloseFeature extends TextDocumentNotificationFeature {
    constructor(client, environmentTextDocument) {
        super(client, environmentTextDocument.pipe(bufferCount(2, 1), 
        // When the previous value emitted was a document, it means the most recent value was either a
        // different document or null. In both cases, it means the previous document is closed.
        filter((v) => v[0] !== null), map(([closedDocument]) => closedDocument)), DidCloseTextDocumentNotification.type, client.options.middleware.didClose, textDocument => ({
            textDocument: { uri: textDocument.uri },
        }), match);
    }
    get messages() {
        return DidCloseTextDocumentNotification.type;
    }
    fillClientCapabilities(capabilities) {
        ensure(ensure(capabilities, 'textDocument'), 'synchronization').dynamicRegistration = true;
    }
    initialize(capabilities, documentSelector) {
        const textDocumentSyncOptions = resolveTextDocumentSync(capabilities.textDocumentSync);
        if (documentSelector && textDocumentSyncOptions && textDocumentSyncOptions.openClose) {
            this.register(this.messages, {
                id: uuidv4(),
                registerOptions: { documentSelector },
            });
        }
    }
}
export function resolveTextDocumentSync(value) {
    if (!value) {
        return undefined;
    }
    else if (typeof value === 'number') {
        return {
            openClose: true,
            change: value,
            save: { includeText: false },
        };
    }
    return value;
}
//# sourceMappingURL=textDocument.js.map