import { BehaviorSubject, Subject, Subscription } from 'rxjs';
import { Client } from '../client/client';
import { ExecuteCommandFeature } from '../client/features/command';
import { ConfigurationChangeNotificationFeature, ConfigurationFeature, ConfigurationUpdateFeature, } from '../client/features/configuration';
import { ContextFeature } from '../client/features/context';
import { ContributionFeature } from '../client/features/contribution';
import { TextDocumentDecorationFeature } from '../client/features/decoration';
import { TextDocumentHoverFeature } from '../client/features/hover';
import { TextDocumentDefinitionFeature, TextDocumentImplementationFeature, TextDocumentReferencesFeature, TextDocumentTypeDefinitionFeature, } from '../client/features/location';
import { WindowLogMessageFeature } from '../client/features/logMessage';
import { WindowShowMessageFeature } from '../client/features/message';
import { TextDocumentDidCloseFeature, TextDocumentDidOpenFeature } from '../client/features/textDocument';
import { isEqual } from '../util';
import { applyContextUpdate, EMPTY_CONTEXT } from './context/context';
import { createObservableEnvironment, EMPTY_ENVIRONMENT } from './environment';
import { Registries } from './registries';
/**
 * The controller for the environment.
 *
 * @template X extension type
 * @template C configuration cascade type
 */
export class Controller {
    constructor(options) {
        this.options = options;
        this._environment = new BehaviorSubject(EMPTY_ENVIRONMENT);
        this._clientEntries = new BehaviorSubject([]);
        this.subscriptions = new Subscription();
        this._logMessages = new Subject();
        this._showMessages = new Subject();
        this._showMessageRequests = new Subject();
        this._showInputs = new Subject();
        this._configurationUpdates = new Subject();
        /** Log messages from extensions. */
        this.logMessages = this._logMessages;
        /** Messages from extensions intended for display to the user. */
        this.showMessages = this._showMessages;
        /** Messages from extensions requesting the user to select an action. */
        this.showMessageRequests = this._showMessageRequests;
        /** Messages from extensions requesting text input from the user. */
        this.showInputs = this._showInputs;
        /** Configuration updates from extensions. */
        this.configurationUpdates = this._configurationUpdates;
        /**
         * Detect when setEnvironment is called within a setEnvironment call, which probably means there is a bug.
         */
        this.inSetEnvironment = false;
        this.environment = createObservableEnvironment(this._environment);
        this.subscriptions.add(() => {
            for (const c of this._clientEntries.value) {
                c.client.unsubscribe();
            }
        });
        this.registries = new Registries(this.environment);
    }
    /** An observable that emits whenever the set of clients managed by this controller changes. */
    get clientEntries() {
        return this._clientEntries;
    }
    setEnvironment(nextEnvironment) {
        if (this.inSetEnvironment) {
            throw new Error('setEnvironment may not be called recursively');
        }
        this.inSetEnvironment = true;
        if (this.options.environmentFilter) {
            nextEnvironment = this.options.environmentFilter(nextEnvironment);
        }
        // External consumers don't see context, and their setEnvironment args lack context.
        if (nextEnvironment.context === EMPTY_CONTEXT) {
            nextEnvironment = Object.assign({}, nextEnvironment, { context: this._environment.value.context });
        }
        if (isEqual(this._environment.value, nextEnvironment)) {
            this.inSetEnvironment = false;
            return; // no change
        }
        this._environment.next(nextEnvironment);
        this.onEnvironmentChange();
        this.inSetEnvironment = false;
    }
    onEnvironmentChange() {
        const environment = this._environment.value; // new environment
        // Diff clients.
        const newClients = computeClients(environment);
        const nextClients = [];
        const unusedClients = [];
        for (const oldClient of this._clientEntries.value) {
            const newIndex = newClients.findIndex(newClient => isEqual(oldClient.key, newClient));
            if (newIndex === -1) {
                // Client is no longer needed.
                unusedClients.push(oldClient);
            }
            else {
                // Client already exists. Settings may have changed, but ConfigurationFeature is responsible for
                // notifying the server of configuration changes.
                newClients.splice(newIndex, 1);
                nextClients.push(oldClient);
            }
        }
        // Remove clients that are no longer in use.
        for (const unusedClient of unusedClients) {
            unusedClient.client.unsubscribe();
        }
        // Create new clients.
        for (const key of newClients) {
            // Find the extension that this client is for.
            const extension = environment.extensions.find(x => x.id === key.id);
            if (!extension) {
                throw new Error(`extension not found: ${key.id}`);
            }
            // Construct client.
            const clientOptions = {
                root: key.root,
                documentSelector: ['*'],
                createMessageTransports: null,
            };
            Object.assign(clientOptions, this.options.clientOptions(key, clientOptions, extension));
            const client = new Client(key.id, key.id, clientOptions);
            // Register client features.
            this.registerClientFeatures(client, this.environment.configuration);
            // Activate client.
            client.activate();
            nextClients.push({
                key,
                client,
            });
        }
        this._clientEntries.next(nextClients);
    }
    registerClientFeatures(client, configuration) {
        client.registerFeature(new ConfigurationChangeNotificationFeature(client, configuration));
        client.registerFeature(new ConfigurationFeature(client, configuration));
        client.registerFeature(new ConfigurationUpdateFeature(client, (params) => new Promise(resolve => this._configurationUpdates.next(Object.assign({}, params, { extension: client.id, resolve })))));
        client.registerFeature(new ContextFeature(client, (params) => 
        // Set environment manually, not via Controller#setEnvironment, to avoid recursive setEnvironment calls
        // (when this callback is called during setEnvironment's teardown of unused clients).
        this._environment.next(Object.assign({}, this._environment.value, { context: applyContextUpdate(this._environment.value.context, params.updates) }))));
        client.registerFeature(new ContributionFeature(this.registries.contribution));
        client.registerFeature(new ExecuteCommandFeature(client, this.registries.commands));
        client.registerFeature(new TextDocumentDidOpenFeature(client, this.environment.textDocument));
        client.registerFeature(new TextDocumentDidCloseFeature(client, this.environment.textDocument));
        client.registerFeature(new TextDocumentDefinitionFeature(client, this.registries.textDocumentDefinition));
        client.registerFeature(new TextDocumentImplementationFeature(client, this.registries.textDocumentImplementation));
        client.registerFeature(new TextDocumentReferencesFeature(client, this.registries.textDocumentReferences));
        client.registerFeature(new TextDocumentTypeDefinitionFeature(client, this.registries.textDocumentTypeDefinition));
        client.registerFeature(new TextDocumentHoverFeature(client, this.registries.textDocumentHover));
        client.registerFeature(new TextDocumentDecorationFeature(client, this.registries.textDocumentDecoration));
        client.registerFeature(new WindowLogMessageFeature(client, (params) => this._logMessages.next(Object.assign({}, params, { extension: client.id }))));
        client.registerFeature(new WindowShowMessageFeature(client, (params) => this._showMessages.next(Object.assign({}, params, { extension: client.id })), (params) => new Promise(resolve => {
            this._showMessageRequests.next(Object.assign({}, params, { extension: client.id, resolve }));
        }), (params) => new Promise(resolve => {
            this._showInputs.next(Object.assign({}, params, { extension: client.id, resolve }));
        })));
    }
    set trace(value) {
        for (const client of this._clientEntries.value) {
            client.client.trace = value;
        }
    }
    unsubscribe() {
        this.subscriptions.unsubscribe();
    }
}
function computeClients(environment) {
    const clients = [];
    if (!environment.extensions) {
        return clients;
    }
    for (const x of environment.extensions) {
        clients.push({ id: x.id, root: environment.root });
    }
    return clients;
}
//# sourceMappingURL=controller.js.map