import { combineLatest, from } from 'rxjs';
import { map, switchMap } from 'rxjs/operators';
import { FeatureProviderRegistry } from './registry';
import { flattenAndCompact } from './util';
/** Provides location results from all extensions for definition, implementation, and type definition requests. */
export class TextDocumentLocationProviderRegistry extends FeatureProviderRegistry {
    getLocation(params) {
        return getLocation(this.providers, params);
    }
}
/**
 * Returns an observable that emits all providers' location results whenever any of the last-emitted set of
 * providers emits hovers.
 *
 * Most callers should use the TextDocumentLocationProviderRegistry class, which uses the registered providers.
 */
export function getLocation(providers, params) {
    return getLocations(providers, params).pipe(map(results => {
        if (results !== null && results.length === 1) {
            return results[0];
        }
        return results;
    }));
}
/**
 * Like getLocation, except the returned observable never emits singular values, always either an array or null.
 */
export function getLocations(providers, params) {
    return providers.pipe(switchMap(providers => {
        if (providers.length === 0) {
            return [null];
        }
        return combineLatest(providers.map(provider => from(provider(params))));
    }), map(flattenAndCompact));
}
/**
 * Provides reference results from all extensions.
 *
 * Reference results are always an array or null, unlike results from other location providers (e.g., from
 * textDocument/definition), which can be a single item, an array, or null.
 */
export class TextDocumentReferencesProviderRegistry extends TextDocumentLocationProviderRegistry {
    /** Gets reference locations from all extensions. */
    getLocation(params) {
        // References are always an array (unlike other locations, which can be returned as L | L[] |
        // null).
        return getLocations(this.providers, params);
    }
}
//# sourceMappingURL=location.js.map