import { Subscription } from 'rxjs';
import { createMessageConnection } from '../jsonrpc2/connection';
import { InitializedNotification, InitializeRequest, } from '../protocol';
import { createExtCommands } from './features/commands';
import { createExtConfiguration } from './features/configuration';
import { createExtContext } from './features/context';
import { createExtWindows } from './features/windows';
class ExtensionHandle {
    constructor(rawConnection, initializeParams) {
        this.rawConnection = rawConnection;
        this.initializeParams = initializeParams;
        this.subscription = new Subscription();
        this.subscription.add(this.rawConnection);
        this.configuration = createExtConfiguration(this, initializeParams.configurationCascade);
        this.windows = createExtWindows(this);
        this.commands = createExtCommands(this);
        this.context = createExtContext(this);
    }
    get root() {
        return this.initializeParams.root;
    }
    close() {
        this.subscription.unsubscribe();
    }
}
const consoleLogger = {
    error(message) {
        console.error(message);
    },
    warn(message) {
        console.warn(message);
    },
    info(message) {
        console.info(message);
    },
    log(message) {
        console.log(message);
    },
};
/**
 * Activates a CXP extension by calling its `run` entrypoint function with the CXP API handle as the first
 * argument.
 *
 * @template C the extension's settings
 * @param transports The message reader and writer to use for communication with the client.
 * @param run The extension's `run` entrypoint function.
 * @return A promise that resolves when the extension's `run` function has been called.
 */
export function activateExtension(transports, run) {
    const connection = createMessageConnection(transports, consoleLogger);
    return new Promise(resolve => {
        let initializationParams;
        connection.onRequest(InitializeRequest.type, params => {
            initializationParams = params;
            return {
                capabilities: {
                    textDocumentSync: { openClose: true },
                    decorationProvider: true,
                },
            };
        });
        connection.onNotification(InitializedNotification.type, () => {
            run(new ExtensionHandle(connection, initializationParams));
            resolve();
        });
        connection.listen();
    });
}
//# sourceMappingURL=extension.js.map