import { isFunction } from '../../util';
import { Emitter } from '../events';
/**
 * A mock implementation of {@link MessageConnection} for use in tests.
 */
export class MockMessageConnection {
    constructor() {
        /**
         * Messages sent by calls to {@link MockMessageConnection#sendRequest} or
         * {@link MockMessageConnection#sendNotification}.
         */
        this.sentMessages = [];
        /**
         * Message handlers registered by calls to {@link MockMessageConnection#onRequest} or
         * {@link MockMessageConnection#onNotification}.
         */
        this.registeredHandlers = new Map();
        /**
         * Mock responses that are returned as results from {@link MockMessageConnection#sendRequest}.
         */
        this.mockResults = new Map();
        this.errorEmitter = new Emitter();
        this.onError = this.errorEmitter.event;
        this.closeEmitter = new Emitter();
        this.onClose = this.closeEmitter.event;
        this.unhandledNotificationEmitter = new Emitter();
        this.onUnhandledNotification = this.unhandledNotificationEmitter.event;
        this.unsubscribeEmitter = new Emitter();
        this.onUnsubscribe = this.unsubscribeEmitter.event;
    }
    recvRequest(type, params) {
        const method = typeof type === 'string' ? type : type.method;
        const handler = this.registeredHandlers.get(method);
        if (!handler) {
            throw new Error(`no mock request handler for method ${method}`);
        }
        return Promise.resolve(handler(params));
    }
    recvNotification(type, params) {
        const method = typeof type === 'string' ? type : type.method;
        const handler = this.registeredHandlers.get(method);
        if (!handler) {
            throw new Error(`no mock notification handler for method ${method}`);
        }
        handler(params);
    }
    sendRequest(type, params, token) {
        const method = typeof type === 'string' ? type : type.method;
        this.sentMessages.push({ method, params });
        if (!this.mockResults.has(method)) {
            throw new Error(`no mock result for method ${method}`);
        }
        const resultOrError = this.mockResults.get(method);
        return resultOrError instanceof Error ? Promise.reject(resultOrError) : Promise.resolve(resultOrError);
    }
    sendNotification(type, params) {
        this.sentMessages.push({ method: typeof type === 'string' ? type : type.method, params });
    }
    onRequest(arg1, arg2) {
        return this.onHandler(arg1, arg2);
    }
    onNotification(arg1, arg2) {
        return this.onHandler(arg1, arg2);
    }
    onHandler(arg1, arg2) {
        let method;
        let handler;
        if (isFunction(arg1)) {
            method = '*';
            handler = arg1;
        }
        else {
            method = typeof arg1 === 'string' ? arg1 : arg1.method;
            handler = arg2;
        }
        this.registeredHandlers.set(method, handler);
    }
    trace(value, tracer, sendNotification) {
        /* noop */
    }
    listen() {
        /* noop */
    }
    unsubscribe() {
        /* noop */
    }
}
//# sourceMappingURL=mockMessageConnection.js.map