import { AbstractMessageReader, AbstractMessageWriter } from '../transport';
class WebSocketMessageReader extends AbstractMessageReader {
    constructor(socket) {
        super();
        this.socket = socket;
        this.pending = [];
        this.callback = null;
        socket.addEventListener('message', (e) => {
            try {
                this.processMessage(e);
            }
            catch (err) {
                this.fireError(err);
            }
        });
        socket.addEventListener('error', err => this.fireError(err));
        socket.addEventListener('close', () => this.fireClose());
    }
    processMessage(e) {
        const message = JSON.parse(e.data);
        if (this.callback) {
            this.callback(message);
        }
        else {
            this.pending.push(message);
        }
    }
    listen(callback) {
        if (this.callback) {
            throw new Error('callback is already set');
        }
        this.callback = callback;
        while (this.pending.length !== 0) {
            callback(this.pending.pop());
        }
    }
    unsubscribe() {
        super.unsubscribe();
        this.callback = null;
        closeIfOpen(this.socket);
    }
}
class WebSocketMessageWriter extends AbstractMessageWriter {
    constructor(socket) {
        super();
        this.socket = socket;
        this.errorCount = 0;
        socket.addEventListener('close', () => this.fireClose());
    }
    write(message) {
        try {
            this.socket.send(JSON.stringify(message));
        }
        catch (err) {
            this.fireError(err, message, ++this.errorCount);
        }
    }
    unsubscribe() {
        super.unsubscribe();
        closeIfOpen(this.socket);
    }
}
function closeIfOpen(socket) {
    if (socket.readyState === socket.OPEN) {
        // 1000 means normal closure. See
        // https://www.iana.org/assignments/websocket/websocket.xml#close-code-number.
        socket.close(1000);
    }
}
/** Creates JSON-RPC2 message transports for a WebSocket. */
export function createWebSocketMessageTransports(socket) {
    const transports = {
        reader: new WebSocketMessageReader(socket),
        writer: new WebSocketMessageWriter(socket),
    };
    if (socket.readyState === socket.OPEN) {
        return Promise.resolve(transports);
    }
    return new Promise((resolve, reject) => {
        function cleanup() {
            socket.removeEventListener('open', doResolve);
            socket.removeEventListener('close', doReject);
            socket.removeEventListener('error', doReject);
        }
        function doResolve() {
            cleanup();
            resolve(transports);
        }
        function doReject(err) {
            cleanup();
            reject(err);
        }
        socket.addEventListener('open', doResolve);
        socket.addEventListener('close', doResolve);
        socket.addEventListener('error', doResolve);
    });
}
//# sourceMappingURL=browserWebSocket.js.map