import { AbstractMessageReader, AbstractMessageWriter } from '../transport';
class WebSocketMessageReader extends AbstractMessageReader {
    constructor(socket) {
        super();
        this.socket = socket;
        this.pending = [];
        this.callback = null;
        socket.on('message', data => {
            try {
                this.processMessage(data);
            }
            catch (err) {
                this.fireError(err);
            }
        });
        socket.on('error', err => this.fireError(err));
        socket.on('close', () => this.fireClose());
    }
    processMessage(data) {
        const message = JSON.parse(data.toString());
        if (this.callback) {
            this.callback(message);
        }
        else {
            this.pending.push(message);
        }
    }
    listen(callback) {
        if (this.callback) {
            throw new Error('callback is already set');
        }
        this.callback = callback;
        while (this.pending.length !== 0) {
            callback(this.pending.pop());
        }
    }
    unsubscribe() {
        super.unsubscribe();
        this.callback = null;
        closeIfOpen(this.socket);
    }
}
class WebSocketMessageWriter extends AbstractMessageWriter {
    constructor(socket) {
        super();
        this.socket = socket;
        this.errorCount = 0;
        socket.on('close', () => this.fireClose());
    }
    write(message) {
        try {
            this.socket.send(JSON.stringify(message), err => {
                if (err) {
                    this.fireError(err, message, ++this.errorCount);
                }
            });
        }
        catch (err) {
            this.fireError(err, message, ++this.errorCount);
        }
    }
    unsubscribe() {
        super.unsubscribe();
        closeIfOpen(this.socket);
    }
}
function closeIfOpen(socket) {
    if (socket.readyState === socket.OPEN) {
        // 1000 means normal closure. See
        // https://www.iana.org/assignments/websocket/websocket.xml#close-code-number.
        socket.close(1000);
    }
}
/** Creates JSON-RPC2 message transports for a NodeWebSocket. */
export function createWebSocketMessageTransports(socket) {
    const transports = {
        reader: new WebSocketMessageReader(socket),
        writer: new WebSocketMessageWriter(socket),
    };
    if (socket.readyState === socket.OPEN) {
        return Promise.resolve(transports);
    }
    return new Promise((resolve, reject) => {
        socket.prependOnceListener('open', () => {
            socket.off('error', reject);
            resolve(transports);
        });
        socket.prependOnceListener('error', reject);
    });
}
//# sourceMappingURL=nodeWebSocket.js.map