import { throwError } from 'rxjs';
import { first, switchMap } from 'rxjs/operators';
import { ClientState } from '../client/client';
/** Reports the client's current state. */
export function getClientState(client) {
    let clientState;
    client.state
        .pipe(first())
        .subscribe(state => (clientState = state))
        .unsubscribe();
    if (clientState === undefined) {
        // This should never happen, because client.state is implemented by a BehaviorSubject that always has a
        // current value.
        throw new Error('client state is not synchronously available');
    }
    return clientState;
}
/**
 * Returns a Promise that resolves when the client enters ClientState.Active and rejects if it enters a client
 * state that indicates an error.
 */
export function clientStateIsActive(client) {
    return clientStateIs(client, ClientState.Active, [
        ClientState.ActivateFailed,
        ClientState.ShuttingDown,
        ClientState.Stopped,
    ]).then(() => void 0);
}
/**
 * Returns a Promise that resolves when the client enters the given client state and rejects if it enters one of
 * the reject states.
 */
export function clientStateIs(client, resolveState, rejectStates = []) {
    return client.state
        .pipe(switchMap(state => {
        if (state === resolveState) {
            return [state];
        }
        if (rejectStates.includes(state)) {
            return throwError(new Error(`client entered reject state ${ClientState[state]}`));
        }
        return [];
    }), first())
        .toPromise();
}
/** Returns a promise that resolves with the value after the given delay (in milliseconds). */
export function delayed(value, msec) {
    return new Promise(resolve => {
        setTimeout(() => resolve(value), msec);
    });
}
//# sourceMappingURL=helpers.js.map